#ifndef BROMSCRIPT_COMPILER_INCLUDED
#define BROMSCRIPT_COMPILER_INCLUDED

#include "SIF.h"
#include "ByteWriter.h"
#include "Misc.h"

namespace BromScript {
	class Compiler
	{
	public:
		Compiler(Scratch::CString filename, char* chunk, int chunklen);
		Compiler(Scratch::CString filename, char* chunk, int chunklen, bool addheader);
		Compiler(Scratch::CString filename, const char* chunk, int chunklen);
		Compiler(Scratch::CString filename, const char* chunk, int chunklen, bool addheader);
		~Compiler(void);

		byte* Run(int* outbytecodelength);
		void AddDefine(Scratch::CString key, Scratch::CString value);
		void AddSet(Scratch::CString key, Scratch::CString value);

	private:
		Scratch::CString Filename;
		int CurrentLine;
		int CurrentPos;
		bool WriteHeader;

		ByteWriter Writer;

		CDictionary<Scratch::CString, Scratch::CString> Defines;
		CDictionary<Scratch::CString, Scratch::CString> Sets;

		int ChunkSize;
		char* CurrentChunk;

		Scratch::CString* Peekline();
		Scratch::CString* ReadLine();
		Scratch::CString Unescape(Scratch::CString str);

		int SkipIfBlock();
		int DoProcess();
		void DoDefines();

		byte* GetArgumentData(Scratch::CString line, int* size);
		void GetFunctionData(Scratch::CString line, char splitter, List<Scratch::CString>* ret);
		void GetIndexes(Scratch::CString line, List<Scratch::CString>* ret);
		Scratch::CString FindString(Scratch::CString line, Scratch::CString start, Scratch::CString end, int startpos);
		Scratch::CString* FindStringSpecial(Scratch::CString line, Scratch::CString start, Scratch::CString end, int startpos);
		int FindChar(Scratch::CString line, int startpos, char* tofind, int tofindlen);
		
		void ThrowError(Scratch::CString reason);

		void WriteFunction(Scratch::CString line);
		void WriteClass(Scratch::CString line);
		void WriteIf(Scratch::CString line);
		void WriteFor(Scratch::CString line);
		void WriteForeach(Scratch::CString line);
		void WriteWhile(Scratch::CString line);
		void WriteSetVar(Scratch::CString line);
		void WriteReturn(Scratch::CString line);
		void WriteMisc(Scratch::CString line);
	};
}

#endif