#ifndef BROMSCRIPT_INSTANCE_INCLUDED
#define BROMSCRIPT_INSTANCE_INCLUDED

#include "SIF.h"

#include "Variable.h"
#include "CallStack.h"
#include "Function.h"
#include "ByteReader.h"
#include "GarbageCollector.h"
#include "Table.h"
#include "List.h"
#include "Userdata.h"
#include "Compiler.h"

namespace BromScript {
	class Instance
	{
	public:
		static int GetCount();

		Instance();
		~Instance();
		
		Scratch::CString CurrentIncludePath;
		Table* Globals;
		List<CallStack*> CurrentStack;
		List<Userdata*> RegisteredUserdataTypes;

		GarbageCollector GC;
		
		void LoadDefaultLibaries();
		Variable* DoString(const char* chunkname, const char* code);
		Variable* DoString(const char* chunkname, const char* code, int size);
		Variable* DoFile(const char* filename);
		Variable* DoCode(const char* filename, byte* code, int codelen);
		void DoDLL(const char* filename);

		void EnterStack(Function* func);
		void LeaveStack();

		void Error(Scratch::CString msg);

		void SetErrorCallback(ErrorFunction func);
		void RegisterFunction(Scratch::CString key, BSFunction func);
		void RegisterScope(Function* scope);

		Userdata* GetRegisteredUserdata(int typeID);
		Userdata* RegisterUserdata(Scratch::CString name, int typeID, int typesize, BSFunctionCtor ctor, BSFunctionDtor dtor);
		
		void RemoveVar(Scratch::CString key);
		void SetVar(Scratch::CString key, Variable* var);
		Variable* GetVar(Scratch::CString key);

		Function* GetCurrentFunction();
		CallStack* GetCurrentStack();

	private:
		List<Function*> ChunkScopes;
		List<BSModuleFunction> ModuleExits;

		bool SuppressErrors;

		int CurrentStackIndex;

		ErrorFunction ErrorCallback;
	};
}

#endif